/*
 * pCOS sample application for dumping PDF information
 *
 * $Id: dumper.cpp,v 1.14 2013/08/05 12:50:17 rjs Exp $
 */

#include <iostream>

#include "pcos.hpp"

using namespace std;
using namespace pdflib;

namespace
{

const wstring docoptlist = L"requiredmode=minimum";
const wstring globaloptlist = L"";

wstring get_wstring(const pCOS& pcos, const string& utf8_string);
} // end of anonymous namespace

int main(int argc, char **argv)
{
    pCOS p;

    if (argc != 2)
    {
        wcerr << L"usage: dumper <filename>" << endl;
        return 2;
    }

    try
    {
        p.set_option(globaloptlist);

        /*
         * Caution: For simplicity we assume that the program arguments are
         * encoded as UTF-8, which might not be true in all cases!
         */
        const wstring doc_name(get_wstring(p, string(argv[1])));
        const int doc = p.open_document(doc_name, docoptlist);
        if (doc == -1)
        {
            wcerr << L"Error: " << p.get_apiname() << L": " << p.get_errmsg()
                    << endl;
            return 98;
        }

        /* --------- general information (always available) */

        const int pcosmode = static_cast<int>(p.pcos_get_number(doc, L"pcosmode"));

        wcout << L"    File name: " << p.pcos_get_string(doc, L"filename") << endl;

        wcout << L"  PDF version: " << p.pcos_get_string(doc, L"pdfversionstring")
                << endl;

        wcout << L"   Encryption: " << p.pcos_get_string(doc, L"encrypt/description")
                << endl;

        wcout << L"    Master pw: "
                << (p.pcos_get_number(doc, L"encrypt/master") ? L"yes" : L"no")
                << endl;

        wcout << L"      User pw: "
                << (p.pcos_get_number(doc, L"encrypt/user") ? L"yes" : L"no") << endl;

        wcout << L"Attachment pw: "
                << (p.pcos_get_number(doc, L"encrypt/attachment") ? L"yes" : L"no") << endl;

        wcout << L" Text copying: "
                << (p.pcos_get_number(doc, L"encrypt/nocopy") ? L"no" : L"yes")
                << endl;

        wcout << L"   Linearized: "
                << (p.pcos_get_number(doc, L"linearized") ? L"yes" : L"no") << endl;

        if (pcosmode == 0)
        {
            wcerr << L"Minimum mode: no more information available\n\n" << endl;
            p.close_document(doc);
            return 0;
        }

        /* --------- more details (requires at least user password) */

        wcout << L" PDF/X status: " << p.pcos_get_string(doc, L"pdfx") << endl;

        wcout << L" PDF/A status: " << p.pcos_get_string(doc, L"pdfa") << endl;

        wcout << L"     XFA data: " <<
            (static_cast<int>(
                    p.pcos_get_number(doc, L"type:/Root/AcroForm/XFA"))
                                            != pcos_ot_null
                                    ? L"yes" : L"no") << endl;

        wcout << L"   Tagged PDF: "
                << (p.pcos_get_number(doc, L"tagged") ? L"yes" : L"no")
                << endl << endl;

        wcout << L" No. of pages: " << (int) p.pcos_get_number(doc, L"length:pages")
                << endl;

        wcout << L"  Page 1 size: width=" << p.pcos_get_number(doc, L"pages[0]/width")
                << L", height=" << p.pcos_get_number(doc, L"pages[0]/height") << endl;

        int count = static_cast<int>(p.pcos_get_number(doc, L"length:fonts"));
        wcout << L" No. of fonts: " << count << endl;

        int i;
        for (i = 0; i < count; i++)
        {
            wostringstream fonts;

            fonts.str(L"");
            fonts << L"fonts[" << i << L"]/embedded";
            if (p.pcos_get_number(doc, fonts.str()))
                wcout << L"embedded ";
            else
                wcout << L"unembedded ";

            fonts.str(L"");
            fonts << L"fonts[" << i << L"]/type";
            wcout << p.pcos_get_string(doc, fonts.str());

            fonts.str(L"");
            fonts << L"fonts[" << i << L"]/name";
            wcout << L" font " << p.pcos_get_string(doc, fonts.str()) << endl;
        }

        wcout << endl;

        const int plainmetadata =
                static_cast<int>(p.pcos_get_number(doc, L"encrypt/plainmetadata"));

        if (pcosmode == 1 && !plainmetadata
                && (static_cast<int>(p.pcos_get_number(doc, L"encrypt/nocopy"))))
        {
            wcout << L"Restricted mode: no more information available" << endl
                    << endl;
            p.close_document(doc);
            return 0;
        }

        /* --------- document info keys and XMP metadata (requires master pw) */

        count = static_cast<int>(p.pcos_get_number(doc, L"length:/Info"));

        int objtype;
        for (i = 0; i < count; i++)
        {
            wostringstream info;

            info.str(L"");
            info << L"type:/Info[" << i << "]";
            objtype = static_cast<int>(p.pcos_get_number(doc, info.str()));

            info.str(L"");
            info << L"/Info[" << i << "].key";
            wcout.width(13);
            wcout << p.pcos_get_string(doc, info.str()) << L": ";

            /* Info entries can be stored as string or name objects */
            if (objtype == pcos_ot_string || objtype == pcos_ot_name)
            {
                info.str(L"");
                info << L"/Info[" << i << "]";
                wcout << L"'" << p.pcos_get_string(doc, info.str()) << L"'" << endl;
            }
            else
            {
                info.str(L"");
                info << L"type:/Info[" << i << "]";
                wcout << "(" << p.pcos_get_string(doc, info.str()) << " object)"
                        << endl;
            }
        }

        wcout << endl << L"XMP meta data: ";

        objtype = static_cast<int>(p.pcos_get_number(doc, L"type:/Root/Metadata"));
        if (objtype == pcos_ot_stream)
        {
            int len;

            const unsigned char *contents =
                    p.pcos_get_stream(doc, &len, L"", L"/Root/Metadata");

            wcout << len << L" bytes ";

	    /* Demonstrate Unicode conversion */
            const string utf16 = p.convert_to_unicode(L"utf8",
			reinterpret_cast<const char *>(contents),
			L"outputformat=utf16");

            wcout << L"(" << utf16.size() / 2 << L" Unicode characters)";
            wcout << endl << endl;
        }
        else
        {
            wcout << L"not present" << endl << endl;
        }

        p.close_document(doc);
    }

    catch (pCOS::Exception &ex)
    {
        wcerr << L"pCOS exception occurred in dumper:" << endl;
        wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname() << ": "
                << ex.get_errmsg() << endl;
    }

    return 0;
}

namespace
{

/*
 * Get a wstring for the given string, assuming the string is encoded in
 * UTF-8.
 */
wstring get_wstring(const pCOS& pcos, const string& utf8_string)
{
    const size_t size = sizeof(wstring::value_type);
    string wide_string;

    switch (size)
    {
    case 2:
        wide_string = pcos.convert_to_unicode(L"utf8",
			utf8_string, L"outputformat=utf16");
        break;

    case 4:
        wide_string = pcos.convert_to_unicode(L"utf8",
			utf8_string, L"outputformat=utf32");
        break;

    default:
        throw std::logic_error("Unsupported wchar_t size");
    }

    return wstring(reinterpret_cast<const wchar_t *> (wide_string.data()),
            wide_string.length() / size);
}

} // end anonymous namespace
